#!/usr/bin/perl -w

package RPM::Source::Convert::Generic2ALT;
use File::Basename;
use File::Temp qw/tempfile/;
use strict;
use warnings;
use Carp;

our $VERSION=0.18;

#----------
# TODO: think on model...
#----------

sub alt_generic_fix {
    my $spec=shift;
    my @sections=$spec->get_sections();
    my $cleanroot_pattern='^\s*(rm|%{__rm}) -rf (\$RPM_BUILD_ROOT|\%\{?rpm_build_root\}?|\%\{?buildroot\}?)\s*$';
    &alt_generic_check_ahead($spec,$sections[0]);
    &alt_generic_check_zip($spec);
    foreach my $sec (@sections) {
	my $t = $sec->get_type();
	if ($t eq 'package') {
	    $sec->exclude('^(Packager|Vendor|Distribution|Build[Rr]oot):');
	    &alt_generic_cleanup_headers($sec);
	} elsif ($t eq 'description') {
	    &alt_generic_fix_desc($sec);
	} elsif ($t eq 'files') {
	    &alt_generic_cleanup_files($sec);
	} elsif ($t eq 'prep' || $t eq 'build' || $t eq 'install') {
	    $sec->exclude($cleanroot_pattern);
	} elsif ($t eq 'clean') {
	    $sec->exclude($cleanroot_pattern);
	    if ($sec->is_empty()) {
		# ?
	    }
	    # hack! due to unsafe iteration!
	    #$sec->empty();$sec->behead();
	    $spec->del_section('clean');
	} elsif ($t eq 'check') {
	    # %check now supported in alt rpm 5.1+
	    #warn 'TODO: fix %check section properly!';
	    #$spec->del_section('check');
	} elsif ($t eq 'changelog') {
	    unless ($spec->{'CONVERT::GENERIC::NOINFO'}) {
		my ($chfh, $chfilename) = tempfile();
		print $chfh 'EVR: ',scalar $spec->get_evr(),"\n";
		my $mainsec=$spec->get_section('package','');
		for my $tag (qw/Name Version Release Serial Epoch/) {
		    my $value = $mainsec->get_tag($tag);
		    print $chfh $tag, ': ',$value,"\n" if defined $value;
		}
		if ($spec->{'SOURCERPM'}) {
		    print $chfh 'srpm: ',$spec->{'SOURCERPM'},"\n";
		} else {
		    print $chfh 'spec: ',$spec->{'SPECFILE'},"\n";
		}
		print $chfh @{$sec->get_body()};
		close ($chfh);
		$spec->add_source('import.info', FILE=>$chfilename);
	    }
	    $sec->empty();
	} elsif ($t eq 'pre') {
	    &alt_generic_fix_scripts($spec,$sec);
	} elsif ($t eq 'post') {
	    &alt_generic_convert_alternatives($spec,$sec);
	    &alt_generic_fix_scripts($spec,$sec);
	} elsif (($t eq 'postun') or ($t eq 'preun')) {
	    &alt_generic_convert_alternatives($spec,$sec);
	    &alt_generic_fix_scripts($spec,$sec);
	} elsif (($t eq 'triggerin') or ($t eq 'triggerun') or ($t eq 'triggerpostin') or ($t eq 'triggerposun')) {
	    # not good, no state :(
	    #&alt_generic_convert_alternatives($spec,$sec);
	} elsif ($t eq 'pretrans') {
	    # what to do ???
	} elsif ($t eq 'posttrans') {
	    &alt_generic_fix_scripts($spec,$sec);
	}
    }

    # second pass
    foreach my $sec (@sections) {
	my $t = $sec->get_type();
	if ($t eq 'package') {
	    &alt_generic_merge_main_group_if_group_is_missing($spec,$sec);
	}
    }

}

our @ALT_ALLOWED_GROUPS=(
'Accessibility',
'Archiving/Backup',
'Archiving/Cd burning',
'Archiving/Compression',
'Archiving/Other',
'Books/Computer books',
'Books/Faqs',
'Books/Howtos',
'Books/Literature',
'Books/Other',
'Communications',
'Databases',
'Development/C',
'Development/C++',
'Development/Databases',
'Development/Debuggers',
'Development/Documentation',
'Development/Functional',
'Development/GNOME and GTK+',
'Development/Haskell',
'Development/Java',
'Development/KDE and QT',
'Development/Kernel',
'Development/Lisp',
'Development/ML',
'Development/Objective-C',
'Development/Other',
'Development/Perl',
'Development/Python',
'Development/Ruby',
'Development/Scheme',
'Development/Tcl',
'Documentation',
'Editors',
'Education',
'Emulators',
'File tools',
'Games/Adventure',
'Games/Arcade',
'Games/Boards',
'Games/Cards',
'Games/Educational',
'Games/Other',
'Games/Puzzles',
'Games/Sports',
'Games/Strategy',
'Graphical desktop/Enlightenment',
'Graphical desktop/FVWM based',
'Graphical desktop/GNOME',
'Graphical desktop/GNUstep',
'Graphical desktop/Icewm',
'Graphical desktop/KDE',
'Graphical desktop/Motif',
'Graphical desktop/Other',
'Graphical desktop/Rox',
'Graphical desktop/Sawfish',
'Graphical desktop/Window Maker',
'Graphical desktop/XFce',
'Graphics',
'Monitoring',
'Networking/Chat',
'Networking/DNS',
'Networking/File transfer',
'Networking/FTN',
'Networking/IRC',
'Networking/Instant messaging',
'Networking/Mail',
'Networking/News',
'Networking/Other',
'Networking/Remote access',
'Networking/WWW',
'Office',
'Publishing',
'Sciences/Astronomy',
'Sciences/Biology',
'Sciences/Chemistry',
'Sciences/Computer science',
'Sciences/Geosciences',
'Sciences/Mathematics',
'Sciences/Medicine',
'Sciences/Other',
'Sciences/Physics',
'Security/Antivirus',
'Security/Networking',
'Shells',
'Sound',
'System/Base',
'System/Configuration/Boot and Init',
'System/Configuration/Hardware',
'System/Configuration/Networking',
'System/Configuration/Other',
'System/Configuration/Packaging',
'System/Configuration/Printing',
'System/Fonts/Console',
'System/Fonts/True type',
'System/Fonts/Type1',
'System/Fonts/X11 bitmap',
'System/Internationalization',
'System/Kernel and hardware',
'System/Libraries',
'System/Legacy libraries',
'System/Servers',
'System/Servers/ZProducts',
'System/X11',
'System/XFree86',
'Terminals',
'Text tools',
'Toys',
'Video',
);

our %IS_ALLOWED_GROUP = map {$_=>1} @ALT_ALLOWED_GROUPS;

our %ALT_GROUP_MAP = (
# generic game conversion
'Amusements/Games', 'Games/Other',
'Applications/Publishing', 'Publishing',
# in docbook5-style-xsl
'Applications/Text', 'Text tools',
'Text Editors', 'Editors',
'System Environment/Base', 'System/Base',
# in aspell
#'Applications/Text', 'System/Internationalization',
# in eclipse-nls
'Development/Languages', 'System/Internationalization',
' ', ' ',
    ); 

sub alt_generic_cleanup_headers {
    my $section=shift;
    my @out;
    foreach (@{$section->get_body()}) {
	if (/^Group:(\s*)(.+\S)\s*$/) {
	    my $gspace=$1;
	    my $group=$2;
	    if ($ALT_GROUP_MAP{$group}) {
		push @out, "Group:$gspace$ALT_GROUP_MAP{$group}\n";
	    } else {
		push @out, $_;
	    }
	} elsif (/^\s*((?:BuildRequires|PreReq|BuildPreReq|BuildPrereqs|Requires|Conflicts)\s*(?:\(\w+\))?)\s*:(\s*)(.*\S)\s*$/) {
	    my $Tag=$1;
	    my $space=$2;
	    my $filtered = &alt_filter_pkg_req_list($3);
	    push @out, "$Tag:$space$filtered\n" if ($filtered);
	} elsif (/^\s*(Provides)\s*:(\s*)(.*\S)\s*$/) {
	    my $Tag=$1;
	    my $space=$2;
	    my $filtered = &alt_filter_pkg_prov_list($3);
	    push @out, "$Tag:$space$filtered\n" if ($filtered);
	} elsif (/^(Url|Source\d*):(\s*)(.+\S)\s*$/) {
	    my $Tag=$1;
	    my $tspace=$2;
	    my $url = $3;
	    # unquoted space in url hack (common in fedora)
	    $url=~s/(?<!\%)\%20/\%\%20/g;
	    push @out, "$Tag:$tspace$url\n";
	} else {
	    push @out, $_;
	}
    }
    $section->set_body(\@out);
}

our %ALTPROVSKIP = map {$_ => 1} qw!!;
our %ALTPROVSUBST = ();

our %ALTDEPSKIP = map {$_ => 1} qw!
%{__make}
/bin/rm /bin/ln
redhat-lsb
!;

our %ALTDEPSUBST = (
'%{_sbindir}/update-alternatives' => 'alternatives >= 0:0.4',
'/usr/sbin/update-alternatives' => 'alternatives >= 0:0.4',
'%{_sbindir}/alternatives' => 'alternatives >= 0:0.4',
'/usr/sbin/alternatives' => 'alternatives >= 0:0.4',
'adns-devel' => 'libadns-devel',
'alsa-lib' => 'libalsa',
'alsa-lib-devel' => 'libalsa-devel',
'apr-devel' => 'libapr1-devel',
'arts-devel' => 'libarts-devel',
'at-spi' => 'libat-spi',
'at-spi-devel' => 'libat-spi-devel',
'automake14' => 'automake_1.4',
'automake16' => 'automake_1.6',
'automake17' => 'automake_1.7',
'automake18' => 'automake_1.8',
'automake19' => 'automake_1.9',
'cairo-devel' => 'libcairo-devel',
'cairo' => 'libcairo',
'commons-digester' => 'jakarta-commons-digester',
'dejavu-sans-fonts' => 'fonts-ttf-dejavu',
'DevIL-devel' => 'libdevil-devel',
'dynamite-devel' => 'libdynamite-devel',
'enet-devel' => 'libenet-devel',
'flac-devel' => 'libflac-devel',
'flex-static' => 'flex',
'fltk-devel' => 'libfltk-devel',
'freealut-devel' => 'libalut-devel',
'freetype-devel' => 'libfreetype-devel',
'GConf2-devel' => 'libGConf-devel',
'GeoIP-devel' => 'libGeoIP-devel',
'giflib' => 'libungif',
'giflib-devel' => 'libungif-devel',
'glew-devel' => 'libglew-devel',
'glx-utils' => 'xdriinfo glxinfo',
'gnet2-devel' => 'libgnet-devel',
'gnome-vfs2-devel' => 'gnome-vfs-devel',
'gnome-vfs2' => 'gnome-vfs',
'gnu-free-sans-fonts' => 'fonts-ttf-freefont',
'gnutls-devel' => 'libgnutls-devel',
'gsl-devel' => 'libgsl-devel',
'gtk-sharp2-devel' => 'libgtk-sharp2-devel',
'guichan-devel' => 'libguichan-devel',
'hicolor-icon-theme' => 'icon-theme-hicolor',
'kdegames-devel' => 'kde4games-devel',
'kdelibs3-devel' => 'kdelibs-devel',
'kdelibs4-devel' => 'kde4libs-devel',
'liberation-fonts' => 'fonts-ttf-liberation',
'libsigc++-devel' => 'libsigc++2-devel',
'libstdc++-static' => 'libstdc++-devel-static',
'mono-core' => 'mono',
'mesa-libGL-devel' => 'libmesa-devel',
'mesa-libGLU-devel' => 'libmesa-devel',
'mikmod-devel' => 'libmikmod-devel',
'mozilla-nspr-devel' => 'libnspr-devel',
'netpbm-progs' => 'netpbm',
'nspr-devel' => 'libnspr-devel',
'ode-devel' => 'libode-devel',
'ogre-devel' => 'libogre-devel',
'openal-soft-devel' => 'libopenal-devel',
'OpenSceneGraph-devel' => 'libOpenSceneGraph-devel',
'openssl-devel' => 'libssl-devel',
#'php-devel' => 'php5-devel',
#'php-embedded-devel' => 'php5-devel',
'portaudio-devel' => 'libportaudio2-devel',
'pulseaudio-libs-devel' => 'libpulseaudio-devel',
'PyQt' => 'python-module-PyQt4',
'PyXML' => 'python-module-PyXML',
'python2-devel' => 'python-devel',
'python-imaging' => 'python-module-imaging',
'python-imaging-tk' => 'python-module-imaging-tk',
'python-setuptools' => 'python-module-setuptools',
'rb_libtorrent-devel' => 'libtorrent-rasterbar-devel',
'ruby-libs' => 'libruby',
'SDL' => 'libSDL',
'SDL-devel' => 'libSDL-devel',
'SDL-static' => 'libSDL-devel',
'SDL_gfx' => 'libSDL_gfx',
'SDL_gfx-devel' => 'libSDL_gfx-devel',
'SDL_image' => 'libSDL_image',
'SDL_image-devel' => 'libSDL_image-devel',
'SDL_mixer' => 'libSDL_mixer',
'SDL_mixer-devel' => 'libSDL_mixer-devel',
'SDL_net' => 'libSDL_net',
'SDL_net-devel' => 'libSDL_net-devel',
'SDL_Pango' => 'libSDL_pango',
'SDL_Pango-devel' => 'libSDL_pango-devel',
'SDL_ttf' => 'libSDL_ttf',
'SDL_ttf-devel' => 'libSDL_ttf-devel',
'vte-devel' => 'libvte-devel',
'xorg-x11-fonts-misc' => 'fonts-bitmap-misc',
'xorg-x11-fonts-Type1' => 'fonts-type1-xorg',
'xorg-x11-server-Xvfb' => 'xorg-x11-xvfb',
' ' => ' ',
);
#E:    alfont-devel
#E:    alleggl-devel
#E:    allegro-tools
#E:    ClanLib06-devel
#E:    ClanLib1-devel
#E:    ClanLib-devel
#E:    elice
#E:    fltk-fluid
#E:    fontpackages-devel
#E:    glyph-keeper-devel
#E:    gnu-free-sans-fonts
#E:    gstream-devel
#E:    lib3ds-devel
#E:    libclaw-devel
#E:    libtool-ltdl-devel
#E:    ocaml-camlimages-devel
#E:    ocaml-labltk-devel
#E:    ode-devel
#E:    openal-devel
#E:    PragmARC-devel
#E:    qtiocompressor-devel
#E:    vdr-devel


sub alt_filter_pkg_prov_list {
    $_=$_[0];
    # hack around rpm4/add_changelog error
    #error: Dependency tokens must not contain '%<=>' symbols: Requires: icu4j-eclipse > = 3.6.1
    s/\>\s+=/>=/g;
    s/\<\s+=/<=/g;
    s/\%\{dist\}//g;
    my @in = split /[\s,]+/, $_;
    my @out;
    while (@in) {
	$_ = shift @in;
	if ($ALTPROVSKIP{$_}) {
	    &read_tag_version(\@in);
	    next;
	} elsif ($ALTPROVSUBST{$_}) {
	    push @out, $ALTDEPSUBST{$_};
	} elsif (/^perl\((.+)\)$/) {
	    my $perlreq=$1;
	    if ($perlreq!~/.pm$/) {
		# looks like fedora perl() provides
		$perlreq=~s!::!/!g;
		$perlreq.='.pm';
		$_='perl('.$perlreq.')';
	    }
	    push @out, $_;
	} else {
	    push @out, $_;
	}
    }
    return join(" ",@out);
};

sub alt_filter_pkg_req_list {
    $_=$_[0];
    # hack around rpm4/add_changelog error
    #error: Dependency tokens must not contain '%<=>' symbols: Requires: icu4j-eclipse > = 3.6.1
    s/\>\s+=/>=/g;
    s/\<\s+=/<=/g;
    s/\%\{dist\}//g;
    my @in = split /[\s,]+/, $_;
    my @out;
    while (@in) {
	$_ = shift @in;
	if ($ALTDEPSKIP{$_}) {
	    &read_tag_version(\@in);
	    next;
	} elsif ($ALTDEPSUBST{$_}) {
	    push @out, $ALTDEPSUBST{$_};
	} elsif (/^perl\((.+)\)$/) {
	    my $perlreq=$1;
	    if ($perlreq!~/.pm$/) {
		# looks like fedora perl() provides
		$perlreq=~s!::!/!g;
		$perlreq.='.pm';
		$_='perl('.$perlreq.')';
	    }
	    push @out, $_;
	} else {
	    push @out, $_;
	}
    }
    return join(" ",@out);
};


# is macro %name used in spec before Name:,Version: tags
# it is not allowed in alt
sub alt_generic_check_ahead {
    my ($spec,$section)=@_;
    my @out;
    my $is_name_passed=0;
    my $is_name_ahead=0;
    foreach (@{$section->get_body()}) {
	m/^\s*Name:/ and $is_name_passed=1; #and print STDERR "passed $_\n";
	$is_name_ahead=1 if m/\%\{name\}/ and $is_name_passed==0;
#print STDERR "AHEAD $_\n";
	$is_name_ahead=1 if m/\%name(?:[^\w\d_]|$)/ and $is_name_passed==0;
#print STDERR "ahead $_\n";
    }
    if ($is_name_ahead) {
	$section->unshift_body('%define name '.$spec->get_section('package','')->get_tag('Name')."\n");
	$section->unshift_body('%define version '.$spec->get_section('package','')->get_tag('Version')."\n");
	print STDERR "fixed ahead!!!\n";
    }
}

sub alt_generic_fix_desc {
    my $section=shift;
    my @out;
    foreach (@{$section->get_body()}) {
# some distro use iso in description. alt use ascii only.
	    #tr//e/;
	    tr/ޜ/...eooeaiaoucououaaidieaayvnieouabEOOEAIAOUCOUOUAAIDIEAAYVNIEOUAB''/;
	    # % in description
	    s,%(\s|$),%% ,g;
	    push @out, $_;
    }
    $section->set_body(\@out);
}

sub alt_generic_check_zip {
    my ($spec)=@_;
    my $zip_found=0;
    foreach my $srcitem (@{$spec->{SOURCES}}) {
	$zip_found=1 if $srcitem and $srcitem=~/\.zip$/i;
    }
    if ($zip_found) {
	my $mainsec=$spec->get_section('package','');
	$mainsec->unshift_body(q!# one of the sources is a zip file
BuildRequires: unzip
!);
    }
}

#error: Group field must be present in package: <subpackage>
sub alt_generic_merge_main_group_if_group_is_missing { 
    my ($spec,$section)=@_;
    my $group=$section->get_tag('Group');
    unless ($group) {
	my $mainsec=$spec->get_section('package','');
	my $maingroup=$section->get_tag('Group');
	die "FATAL: main section does not contain a Group: tag" unless $maingroup;
	$section->set_tag('Group',$maingroup);
    }
    
}

sub alt_generic_cleanup_files {
    my $section=shift;
    my %docdir;
    my @out;
    foreach (@{$section->get_body()}) {
	s/^\s*\%attr\(-,root,root\)\s*//;
	#if (/^\s*\%defattr\(0644,root,root,0755\)\s*$/
	if (/^\s*\%defattr\((?:0644|-),root,root,(?:0755|-)\)\s*$/
	    or /^\s*\%defattr\((?:0664|-),root,root,(?:0755|-)\)\s*$/
	    or /^\s*\%defattr\((?:0644|-),root,root\)\s*$/
	    ) {
	    # throw away
	} elsif (/^\s*\%defattr\(0755,root,root,0755\)\s*$/) {
	    # dangerous: as in antlr may cause lost 755 rights in /usr/bin
	    push @out, "#".$_;
	} elsif (/^\s*\%doc\s*\%\{_docdir\}\s*$/) {
	    push @out, '%doc %{_docdir}/*'."\n";
	} elsif (m!^\s*\%doc\s*(\%\{?_docdir\}?/\%\{?name\}?-\%\{?version\}?/\S+)!) {
	    &__add_docdir(\%docdir,$1);
	    push @out, $_;
	} elsif (m!^\s*(?:\%doc\s*)?(\%\{?_datadir\}?/doc/\%\{?name\}?-\%\{?version\}?/\S+)!) {
	    &__add_docdir(\%docdir,$1);
	    push @out, $_;
	} elsif (m!^\s*(\%\{?_docdir\}?/\%\{?name\}?-\%\{?version\}?/\S+)!) {
	    &__add_docdir(\%docdir,$1);
	    # note: no need to add a %doc here, rpm will add it for us.
	    push @out, $_;
	} else {
	    push @out, $_;
	}
    }
    $section->set_body(\@out);
    if (scalar %docdir) {
	$section->push_body(
'# hack; explicitly added docdir if not owned
'.join ('',map {'%doc %dir '.$_."\n"} keys (%docdir))
);
    }
}

sub __add_docdir {
    my ($dochashptr,$dir) =@_;
    $dir=~s/\%name-\%version/%{name}-%{version}/;
    $dir=~s/\%_docdir/%{_docdir}/;
    $dir=~s!\%{_datadir}/doc!%{_docdir}!;
    $dir=~s!\%_datadir/doc!%{_docdir}!;
    $dochashptr->{dirname($dir)}=1;
    $dochashptr->{'%{_docdir}/%{name}-%{version}'}=1;
}

sub alt_generic_fix_scripts {
    my $spec=shift;
    my $section=shift;
    my $shell=$section->get_flag('-p');
    if (defined $shell and $shell=~/ldconfig/) {
	print STDERR 'Generic2ALT: deleted %'.$section->get_type()." ".$section->get_package()." -p *ldconfig*\n";
	$spec->del_section($section->get_type(),$section->get_package(),$section->get_trigger_condition());
	return;
    }
    $section->subst(qr'/sbin/ldconfig',': ');


    # obsolete-call-in-post-gtk-update-icon-cache.pl (repocop-unittest-rpm-filetriggers-0.20-alt7)
    $section->exclude(qr'^\s*touch\s+--no-create\s+(\%\{?_datadir\}?/icons|\%\{?_iconsdir\}?)/hicolor');
    $section->exclude_multi(
	qr'^\s*if\s+\[\s+-x\s+.*gtk-update-icon-cache\s+\]\s*;\s*then',
	qr'^\s*.*gtk-update-icon-cache',
	qr'^\s*fi',
	);
    $section->exclude_multi(
	qr'^\s*\[\s+"?\$1"?\s+-eq\s+1\s+\] &&\s+\\$',
	qr'^\s*.*gtk-update-icon-cache',
	);
    my @body=@{$section->get_body()};
    my @new_body;
    my $multiline_gtk_update_icon_cache=0;
    my $multiline=0;
    while (scalar @body) {
	$_=shift(@body);
	next if $multiline and /\\\s*$/;
	if ($multiline and not /\\\s*$/) {
	    $multiline=0;
	    next;
	}
	if (/gtk-update-icon-cache/) {
	    $multiline=1 if /\\\s*$/;
	    $multiline_gtk_update_icon_cache=1;
	    next;
	}
	push @new_body, $_;
    }
    if ($multiline_gtk_update_icon_cache) {
	$section->set_body(\@new_body);
	$section->exclude(qr'gtk-update-icon-cache');
    }
    if ($shell && $shell=~/gtk-update-icon-cache/) {
	$spec->del_section($section);
	return;
    }
    #---------------------------------------------------------------------

    if ($section->is_empty()) {
	print STDERR 'Generic2ALT: deleted empty %'.$section->get_type()." ".$section->get_package()."\n";
	$spec->del_section($section->get_type(),$section->get_package(),$section->get_trigger_condition());
    }
                                                                              

}

sub alt_generic_convert_alternatives {
    my $spec=shift;
    my $section=shift;
    my @out;
    my $ptr = $section->get_body();
    while (@$ptr>0) {
	my $line = shift @$ptr;
	if ($line=~/\s*#/) {
	    push @out, $line;
	} elsif ($line=~s,^(\s*)(?:(?:\%{_sbindir}|/usr/sbin)/)?(?:update-)?(?:\%\{__)?alternatives\}? --(\w+),,) {
	    my $preline=$1;
	    my $updcmd=$2;
	    push @out, &_convert_single_alternative($spec,$section,$ptr,$preline,$updcmd,$line);
	} elsif ($line=~s,^(\s*)(?:(?:\%{_sbindir}|/usr/sbin)/)?(?:update-)?(?:\%\{__)?alternatives\}?\s+\\\n$,,) {
	    my $preline=$1;
	    $line = shift @$ptr;
	    if ($line=~s/^\s*--(\w+)//) {
		my $updcmd=$1;
		push @out, &_convert_single_alternative($spec,$section,$ptr,$preline,$updcmd,$line);
	    } else {
		unshift @$ptr, $line;
		push @out, $preline."# unknown update-alternatives\n";
	    }
	} else {
	    push @out, $line;
	}
    }
    $section->set_body(\@out);
}

sub _convert_single_alternative {
    my ($spec,$section,$ptr,$preline,$updcmd,$line)=@_;
    $line=&read_multiline($line, $ptr);
    # the end is abandoned :(
    if ($updcmd eq "remove") {
	my $name=&read_shell_token(\$line);
	# pre 0.4 alt alternatives
	#return $preline.'%unregister_alternatives '.&get_alternatives_file_name($spec,$section,$name)."\n";
	return $preline.":\n";
    } elsif ($updcmd eq "install") {
	my $link=&read_shell_token(\$line);
	my $name=&read_shell_token(\$line);
	my $path=&read_shell_token(\$line);
	my $priority=&read_shell_token(\$line);
	unless ($priority) {
	    $priority=10;
	    warn "alternative priority is 0 - set to $priority\n";
	}
	my @alternatives=($link,$path,$priority);
	&read_space(\$line);
	while ($line=~/--slave/) {
	    &read_shell_token(\$line);
	    my $slink=&read_shell_token(\$line);
	    my $sname=&read_shell_token(\$line);
	    my $spath=&read_shell_token(\$line);
	    push @alternatives, ($slink,$spath,$path);
	}
	&create_alternatives($spec,$section, &get_alternatives_file_name($spec,$section,$name),\@alternatives);
	return $preline.":\n";
    } else {
	warn "unknown update-alternatives --$updcmd";
	return $preline."# unknown update-alternatives --$updcmd\n";
    }
}

sub create_alternatives {
    my ($spec, $section, $alternatives_file_name,$alternativesptr) = @_;
    my @text=('install -d $RPM_BUILD_ROOT/%_altdir; cat >$RPM_BUILD_ROOT/%_altdir/'.$alternatives_file_name.'<<EOF
');
    while (@$alternativesptr>0) {
	my $provided=shift(@$alternativesptr);
	my $alternative=shift(@$alternativesptr);
	my $weight=shift(@$alternativesptr);
	push @text, $provided."\t".$alternative."\t".$weight."\n";
    }
    push @text, "EOF\n";
    $spec->get_section('install')->push_body(@text);
    my $filesec=$spec->get_section('files',$section->get_package());
    if ($filesec) {
	$filesec->unshift_body('%_altdir/'.$alternatives_file_name."\n");
    } else {
	warn "WARNING: section %files for [".$section->get_package()."] not found: can't write alternatives found in ".$section->get_type()."!";    }
}

sub get_alternatives_file_name {
    my ($spec,$section,$name)=@_;
    return $name.'_'.$section->get_package_expanded();
}

sub read_tag_version {
    my $ptr=shift;
    return if ($#{$ptr}<1);
    if ($ptr->[0] =~/^(?:<|>|=|!=|>=|<=)$/) {
	return (shift @$ptr, shift @$ptr);
    }
    return;
}

sub read_multiline {
    my ($line, $ptr) =@_;
    while (@$ptr>0 && $line=~s/\\[\r\n]+?$//) {
	chomp $line;
#	print "MULTILINE:$line\n";
	$line.=shift(@$ptr);
    }
    return $line;
}

sub read_shell_token {
    my ($lineptr) =@_;
    if ($$lineptr=~s/^\s*(\S+)//) {
	return $1;
    }
    return "";
}

sub read_space {
    my ($lineptr) =@_;
    $$lineptr=~s/^\s*//;
}

1;
