#!/usr/bin/perl -w

package RPM::Source::Convert::JPackage2ALT;

use strict;
use warnings;
use Carp;
use RPM::Source::Editor; # for run
use RPM::Source::Convert::Generic2ALT; # for read_tag_version
use File::Basename;

my $TODO=qq~
%if %{native} %if ! %{gcj_support}
BuildRequires:  %{_bindir}/gcj, %{__perl}, 
~;


sub jpp_specific_fix {
    my $spec=shift;
    my $main_sec=$spec->get_section('package','');
    my $name=$main_sec->get_tag('Name');
    $main_sec->unshift_body("BuildRequires: rpm-build-java-osgi\n") if $name=~/^eclipse/;

    if (not $main_sec->match(qr'BuildRequires: jpackage-.+-compat')) {
	my $jppcompat='';
	my $jvm=$spec->{'CONVERT::JPACKAGE::JVM'};
	if ($jvm) {
	    if ($jvm =~ /^4/) {
		$jppcompat='-1.4.2';
	    } elsif ($jvm =~ /^5/) {
		$jppcompat='-1.5.0';
	    } elsif ($jvm =~ /^6/) {
		$jppcompat='-1.6';
	    } elsif ($jvm =~ /^7/) {
		$jppcompat='-generic';
	    } else {
		$jppcompat='';
	    }
	}
	$main_sec->unshift_body("BuildRequires: jpackage$jppcompat-compat\n");
	$main_sec->unshift_body("BuildRequires: /proc\n");
    }

    # maven hack
    if ($spec->{'CONVERT::JPACKAGE::USEMAVENPLAGINS'}) {
	$main_sec->subst_if(qr'\s+maven(\s|$)',' maven-plugins ', qr'^\s*BuildRequires:\s*');
	$main_sec->subst_if(qr'\s+maven2(\s|$)',' maven2-plugins ', qr'^\s*BuildRequires:\s*');
    }

    foreach my $sec ($spec->get_sections()) {
	my $t = $sec->get_type();
	my $pkgname = $sec->get_package();
	#print "type is $t\n";
	if ($t eq 'package') {
	    &jpp_specific_cleanup_headers($sec);
	    &jpp_specific_find_bcond($sec) unless $pkgname;
	    $sec->subst(
		qr'\%define gcj_support \%{\?_with_gcj_support:1}\%{!\?_with_gcj_support:\%{\?_without_gcj_support:0}\%{!\?_without_gcj_support:\%{\?_gcj_support:\%{_gcj_support}}\%{!\?_gcj_support:0}}}', '%define gcj_support 0');
	    $sec->subst(qr'\%define\s+gcj_support\s+1', '%define gcj_support 0');
	    $sec->subst(qr'\%define\s+_with_gcj_support\s+1', '%define _with_gcj_support 0');

	    my $packagename=$sec->get_package();
	    # optimization
	    if ($packagename eq 'javadoc' || $packagename eq 'manual') {
		$sec->push_body('BuildArch: noarch'."\n") unless $sec->match(qr'^BuildArch:\s+noarch');
	    }
	} elsif ($t eq 'files') {
	    &jpp_specific_cleanup_files($sec);
	} elsif ($t eq 'prep' or $t eq 'build' or $t eq 'install') {
	    foreach my $java5pattern (
		qr'export JAVA_HOME=\%{_jvmdir}/java-1.5.0',
		qr'^(?:export\s+)?JAVA_HOME=/usr/lib/jvm/java-1.5.0',
		) {
		if ($sec->match($java5pattern)) {
		$sec->exclude($java5pattern);
		warn "excluded: export JAVA_HOME=/usr/lib/jvm/java-1.5.0\n";
		}
	    }
	} elsif ($t eq 'pre' or $t eq 'post' or $t eq 'preun' or $t eq 'postun') {
	    &jpp_specific_filter_script($sec);
	    if ($sec->is_empty()) {
		print STDERR 'JPackage2ALT: deleted %'.$sec->get_type()." ".$sec->get_package()."\n";
		$spec->del_section($sec->get_type(),$sec->get_package());
	    }
	} elsif ($t eq 'changelog') {
	}
	# general hacks here
	# is it worth wrapping in one?
	$sec->subst(qr'%{_localstatedir}/', '%{_var}/');
	# altlinux vs redhat incompatibility (found in hsqldb)
	#$sec->subst(qr'%{_localstatedir}/lib', '%{_var}/lib');
	#$sec->subst(qr'%{_localstatedir}/log', '%{_var}/log');# jetty 5
	#$sec->subst(qr'%{_localstatedir}/run', '%{_var}/run');# jetty 5
	#$sec->subst(qr'%{_localstatedir}/cache', '%{_var}/cache');# jetty 5
	# due to feature of alt findreq
	$sec->subst(qr'%{_sysconfdir}/init.d','%_initdir');
    }
    # hack... it looks like it should be moved to RPM::Source::Editor
    # zip -> bz2 due to alt buggy rpm (/usr/bin/unzip -qL) - ALREADY FIXED!!!
    $main_sec=$spec->get_section('package','');
    if ($spec->{'CONVERT::JPACKAGE::NOZIP'} and $main_sec->match(qr'^\s*Source\d*:.+\.(zip|jar)\s*$') 
	) {
        # source0 only!! others should be not %setup'ed, or we need explicit check
	#for (my $i=0;$i<@{$spec->{SOURCES}}; $i++) {
	for (my $i=0;$i<1; $i++) {
	    my $source=$spec->{SOURCES}->[$i];
	    $source=~s/\s*$//;
	    if ($source=~/\.(zip|jar)$/ and ! $spec->get_section('prep')->match(qr'unzip -q %{SOURCE'.$i.'\}') and ! $spec->is_specfile_mode()) {
		&_convert_source_from_zip($spec,$source,$i);
	    }
	}
    }
}

sub _convert_source_from_zip {
    my ($spec, $source,$i)=@_;
    print STDERR "note: try to bzip2 zip source [$source]\n";
    my $sourcefile=$spec->{SOURCEDIR}."/".basename($source);
    if (! -e $sourcefile) {
	warn "Internal error: source $source not found!";
    } else {
	my $convertdir= $spec->{TMPDIR}.'/CONVERT'.$i;
	mkdir $convertdir;
	&RPM::Source::Editor::run ("pushd $convertdir; unzip -q $sourcefile; popd");
	my @files= map {s,^\Q$convertdir\E/,,; $_} glob ("$convertdir/*");
	my @hidden_files=grep {!/\.$/ && ! /\*$/} map {s,^\Q$convertdir\E/,,; $_} glob ("$convertdir/.*");
	my $arjsuff='.bz2';
	my $arjtarg='j';
	if ($spec->{'CONVERT::JPACKAGE::NOBZ2'}) {
	    $arjtarg='';
	    $arjsuff='';
	}
	$sourcefile=~s/\.(zip|jar)$/.tar$arjsuff/;
	&RPM::Source::Editor::run ("tar c${arjtarg}f $sourcefile -C $convertdir @files @hidden_files");
	my $rawsource=$spec->{RAWSOURCES}->[$i];
	my $newsource=$rawsource;
	$newsource=~s/\.(zip|jar)$/.tar$arjsuff/;
	my $main_sec=$spec->get_section('package','');
	$main_sec->subst(qr"$rawsource",$newsource);
    }
}

sub jpp_specific_find_bcond {
    my $section=shift;
    my @out;
    foreach (@{$section->get_body()}) {
# at fault?: (jcommon.spec for example)
#%define bcond_with()    %{expand:%%{?_with_%{1}:%%global with_%{1} 1}}
#%define bcond_without() %{expand:%%{!?_without_%{1}:%%global with_%{1} 1}}
	
	if (/^\%bcond_with\s+(\S+)\s*$/) {
	    push @out, '#def_with '.$1."\n";
	    print STDERR 'JPackage2ALT: found disabled (bcond_with): '.$1."\n";
	}
	push @out, $_;
    }
    $section->set_body(\@out);
}


sub jpp_specific_cleanup_files {
    my $section=shift;
    my @out;
    foreach (@{$section->get_body()}) {
	if (/^\s*\%ghost\s*((?:\%dir|\%doc)\s*)*\%{_javadocdir}/) {
	    # we will leave ghost there... a usual symlink will not upgrade from directory
#	    s,^\s*\%ghost\s*,,g;
	    push @out, $_;
#	} elsif (/^\s*\%ghost\s*\%dir\s*\%{_javadocdir}/) {
	    # throw away -- seems dangerous
#	} elsif (/^\s*\%ghost\s*%{_bindir}/) {
	} elsif (/^\s*\%ghost\s*\%{_javadir}/) {
	    # seems to be touch for alternatives. better to exclude.
	    s/^\s*\%ghost\s*/\%exclude /;
	    push @out, $_;
	} elsif (/^\s*\%ghost\s*/) {
	    # alternatives ?
	    push @out, "#".$_;
	} else {
	    push @out, $_;
	}
    }
    $section->set_body(\@out);
}

our %JPP_GROUP_MAP = (
"Editors", "Editors",
"Text Editors", "Editors",
"Development/Tools/IDE", "Editors",
"Utilities", "Utilities",
"Multimedia/Graphics", "Graphics", # batik
"Database", "Databases", # derby
"Applications/Databases", "Databases",
"Networking/Instant messaging", "Networking/Instant messaging", # jext ??? horrible :(
"Internet/Log Analysis", "Monitoring",
"System/Servers", "System/Servers",
"System/Libraries", "System/Libraries", # ht2html
"Text Editors/Integrated Development Environments (IDE)", "Editors",
"System Environment/Base", "System/Base",
"Documentation/HTML", "Documentation",
"Development", "Development/Java",
" ", " ",
    ); 

sub jpp_specific_cleanup_headers {
    my $section=shift;
    my @out;
    foreach (@{$section->get_body()}) {
	if (/^Group:(\s*)(.+\S)\s*$/) {
	    #if ($2) eq "System/Libraries" and ht2html
	    if (0
		or $2 eq "Applications/Text" # adaptx fc10
		or $2 eq "Development/Build Tools"
		or $2 eq "Development/Code Generators"
		or $2 eq "Development/Compilers"
		or $2 eq "Development/Framework"
		or $2 eq "Development/Java"
		or $2 eq "Development/Languages" # ecj
		or $2 eq "Development/Libraries"
		or $2 eq "Development/Libraries/Application Frameworks"
		or $2 eq "Development/LIbraries/Java" # jgroups (bug!)
		or $2 eq "Development/Libraries/Java"
		or $2 eq "Development/libraries/java" # axis2 (bug!)
		or $2 eq "Development/Libraries/java" # axis2 (bug!)
		or $2 eq "Development/Test/Java"
		or $2 eq "Development/Testing"
		or $2 eq "Development/Tools"
		or $2 eq "Development/Frameworks" # richfaces
		or $2 eq "Internet/WWW/Dynamic Content" #tomcat
		or $2 eq "Internet/WWW/Servers" #jboss
		or $2 eq "Internet/WWW/Indexing/Search" # lucene
		or $2 eq "Applications/Internet" # apacheds
		or $2 eq "System/Logging" # log4j
		or $2 eq "System/Networking" # mina
		or $2 eq "System Environment/Applications" #tomcat
		or $2 eq "System Environment/Libraries"
		or $2 eq "Security/Cryptography"
		or $2 eq "Software Development/Quality Assurance"
		or $2 eq "Networking/Daemons" # jacarta-common-daemons
		or $2 eq "System/Boot"
		or $2 eq "Utilities" # jarjar
		or $2 eq "Text Processing/Markup/XML"
		or $2 eq "Text Processing/Markup/HTML"
		or $2 eq "Documentation" # jacorb
		or $2 eq "Development Documentation" # rome
		or $2 eq "Development/Debuggers" # findbugs
) {
		push @out, "Group:${1}Development/Java\n";
	    } elsif ($JPP_GROUP_MAP{$2}) {
		push @out, "Group:${1}$JPP_GROUP_MAP{$2}\n";
	    } elsif ($2 eq "Development/Documentation") {
		push @out, $_;
	    } elsif ($RPM::Source::Convert::Generic2ALT::IS_ALLOWED_GROUP{$2}) {
		push @out, $_;
	    } else {
		die "unknown conversion for Group: $2";
	    }
	} elsif (/^\s*\%define\s+section\s+free\s*$/) {
	    # throw away
	} elsif (/^\s*((?:BuildRequires|PreReq|BuildPreReq|BuildPrereqs|Requires|Conflicts)\s*(?:\(\w+\))?)\s*:\s*(.*\S)\s*$/) {
	    my $Tag=$1;
	    my $filtered = &jpp_filter_pkg_list($2);
	    push @out, "$Tag: $filtered\n" if ($filtered);
	} else {
	    push @out, $_;
	}
    }
    $section->set_body(\@out);
}

our %JPPDEPSKIP = map {$_ => 1} qw!
java-devel java java-sdk
java-1.4.2-devel java-1.5.0-devel java-1.6.0-devel
java-1.4.2 java-1.5.0 java-1.6.0
jce jsse 
java-fonts
!;
# java-devel jce jsse ...  are hack around Conflicts: in jpackage-1.x-compat
#jce jsse 


our %JPPDEPSUBST = (
#'crimson' => 'jakarta-crimson',
#'crimson-demo' => 'jakarta-crimson-demo',
#'crimson-manual' => 'jakarta-crimson-manual',
#'crimson-javadoc' => 'jakarta-crimson-javadoc',
'docbook-xsl-java-saxon' => 'docbook-xsl-saxon',
'smc' => 'jsmc',
'smc-manual' => 'jsmc-manual',
'smc-javadoc' => 'jsmc-javadoc',
'jal' => 'sg-jal',
'jal-javadoc' => 'sg-jal-javadoc',
'java_cup' => 'java-cup',
'maven-modello-plugin' => 'maven-plugin-modello',
# 2.0.7
'maven2-plugin-surefire' => 'maven-surefire-plugin',
'maven2-bootstrap' => 'maven2',
'modello-maven-plugin' => 'maven-plugin-modello',
'mojo-maven2-plugin-changes' => 'maven2-plugin-changes',
'mojo-maven2-plugin-dependency' => 'maven2-plugin-dependency',
'mojo-maven2-plugin-jxr' => 'maven2-plugin-jxr',
#'ant-xslp' => 'ant-optional',
#'' => '',
);

sub jpp_filter_pkg_list {
    my @in = split /[\s,]+/, $_[0];
    my @out;
    while (@in) {
	$_ = shift @in;
	if ($JPPDEPSKIP{$_}) {
	    &RPM::Source::Convert::Generic2ALT::read_tag_version(\@in);
	    next;
	} elsif ($JPPDEPSUBST{$_}) {
	    push @out, $JPPDEPSUBST{$_};
	} else {
	    push @out, $_;
	}
    }
    return join(" ",@out)
}

sub jpp_specific_filter_script {
    my ($section)=@_;
    $section->subst(qr'\%update_maven_depmap',':');

    # below is just a bunch of hacks :(
    &_jpp_filter_script_bin_true($section);
    $section->exclude_multi(
qr'^\s*if\s+\[\s+-x\s+\%{_bindir}/rebuild-gcj-db\s+\]\s*$',
qr'^\s*then\s*$',
#%{_bindir}/rebuild-gcj-db
qr'/rebuild-gcj-db\s*$',
qr'^\s*fi\s*$');
    $section->exclude(qr'^\s*if\s+\[\s+-x\s+\%\{?_bindir\}?/rebuild-gcj-db\s+\];\s*then\s+\%\{?_bindir\}?/rebuild-gcj-db\s*\|\|\s*true\s*;\s*fi\s*$');
    $section->exclude_multi(
qr'^\s*if\s+\[\s+"\$1"\s+=\s+"0"\s+\];\s*then\s*$',
qr'^\s*:\s*$',
qr'^\s*fi\s*$');
#{
#  [ $1 -eq 0 ] || exit 0
#OR  [ $1 = 0 ] || exit 0
#  :
#} >/dev/null 2>&1 || :
    $section->exclude_multi(
qr'^\s*\{\s*$',
qr'^\s*\[\s+"?\$1"?\s+(?:-eq|=)\s+"?0"?\s+\]\s*\|\|\s*exit\s+0\s*$',
qr'^\s*:\s*$',
qr'^\s*\}\s*\>\s*/dev/null\s+2\>\&1\s*\|\|\s*:\s*$');
    # hack again!!!
    $section->set_body(&RPM::Source::Editor::__filter_empty_macro_if_endif($section->get_body()));
    &_jpp_filter_script_bin_true($section);
    print STDERR "dead 6: %",$section->get_type()," ",$section->get_package(),"\n" unless (@{$section->get_body()});
}

sub _jpp_filter_script_bin_true {
    my ($section)=@_;
    my @header;
    my @out;
    my $i = 0;
    my $command_found=0;
    unless (@{$section->get_body()}) {
	print STDERR "strange dead section: %",$section->get_type()," ",$section->get_package(),"\n";
	return;
    }
    push @header, shift @{$section->get_body()};
    foreach (@{$section->get_body()}) {
	if (/^\s*:\s*$/) {
	    push @out, $_ if $i++==0 and $command_found;
	}elsif (/^\s*#/) {
	    push @out, $_;
	}elsif (/^\s*$/) {
	    push @out, $_;
	} else {
	    push @out, $_;
	    #print STDERR "other for ".$section->get_type()," ",$section->get_package(),"\n",$_;
	    $command_found=1;
	    $i=0;
	}
    }
    if ($command_found) {
	$section->set_body([@header,@out]);
	#print STDERR "set cmd for ",@header;
    } else {
	$section->set_body([@header]);
	#print STDERR "Jpackage2ALT: set empty for ",@header;
    }
}

1;
